﻿using System;
using System.Collections;
using Overdash.General;
using Overdash.Global;
using Overdash.Player;
using UnityEngine;

namespace Overdash.Combat.Projectiles
{
    [RequireComponent(typeof(AmmoPickup), typeof(Rigidbody2D), typeof(AudioSource))]
    public abstract class ProjectileBase : MonoBehaviour
    {
        [SerializeField] protected Animator animator;
        [field: SerializeField] public AmmoType ammoType { get; private set; }
        
        public event Action<ProjectilePlayerHit> OnHitPlayer;
        public bool isActive { get; protected set; }
        public Player.Player shooter { get; protected set; }
        public new Rigidbody2D rigidbody { get; private set; }

        public AudioClip wallHitSound;
        public AudioClip playerHitSound;


        private void Awake()
        {
            this.rigidbody = GetComponent<Rigidbody2D>();
            rigidbody.collisionDetectionMode = CollisionDetectionMode2D.Continuous;
            PropsManager.PropsList.Add(gameObject);
            wallHitSound = Resources.Load<AudioClip>("bullet_hit_wall");
            playerHitSound = Resources.Load<AudioClip>("bullet_hit_player");
        }
        
        protected virtual void Update()
        {
            if (isActive)
                rigidbody.velocity = transform.up * ammoType.projectileSpeed;
        }
        
        public virtual void Shoot(Player.Player shooter)
        {
            this.shooter = shooter;
            isActive = true;
            GetComponent<PickupItem>().isPickable = false;
            UpdateAnimator();
        }

        protected virtual void OnCollisionEnter2D(Collision2D collision)
        {
            if (!isActive) 
                return;

            // Player hit
            if (collision.gameObject.TryGetComponent(out PlayerCharacter player))
                PlayerHit(player, collision);
            
            // Wall hit
            else
                WallHit(collision);
        }
        
        protected virtual void PlayerHit(PlayerCharacter player, Collision2D collision) 
        {
            if (player.GetComponent<PlayerMovement>().dashing)
            {
                player.GetComponent<PlayerCombat>().AddAmmunition(ammoType);
                Destroy(gameObject);
            }
            else
            {
                player.Kill(shooter);
                OnHitPlayer?.Invoke(new ProjectilePlayerHit
                {
                    Shooter = shooter,
                    ProjectileBase = this,
                    Collision = collision
                });

                StartCoroutine(DestroyAfterSound(playerHitSound));
            }
            
            
        }

        protected virtual void WallHit(Collision2D _)
        {
            GetComponent<AudioSource>().PlayOneShot(wallHitSound);
            Deactivate();
        }
        
        protected virtual void Deactivate()
        {
            isActive = false;
            rigidbody.constraints = RigidbodyConstraints2D.None;
            GetComponent<PickupItem>().isPickable = true;
            UpdateAnimator();
        }

        protected void InvokeOnHitPlayer(ProjectilePlayerHit hit) => OnHitPlayer?.Invoke(hit);
        
        protected virtual void UpdateAnimator()
        {
            animator.SetBool("isActive", isActive);
        }
        private IEnumerator DestroyAfterSound(AudioClip clip)
        {
            GetComponent<Collider2D>().enabled = false;
            GetComponentInChildren<SpriteRenderer>().enabled = false;

            GetComponent<AudioSource>().PlayOneShot(clip);
            yield return new WaitForSeconds(0.6f);
            Destroy(gameObject);
        }
    }
}